#include "ddraw.h"
#include "DDStuff.h"
#include "DD_ERR.h"
#include "main.h"
#include "son.h"
#include "keyb.h"

LPDIRECTDRAW			lpDD = NULL;
LPDIRECTDRAWSURFACE		lpDDSPrimary = NULL;
LPDIRECTDRAWSURFACE		lpDDSBack = NULL;
LPDIRECTDRAWSURFACE		lpDDSFond = NULL;
LPDIRECTDRAWSURFACE		lpDDSAlphabet = NULL;
LPDIRECTINPUT			lpdi = NULL;
LPDIRECTINPUTDEVICE		lpdiDevice = NULL;
LPDIRECTSOUND           lpds = NULL;
DSBUFFERDESC            dsbdesc;
LPDIRECTSOUNDBUFFER     lpdsbPrimary = NULL;


HRESULT InitDD(HWND &hWnd, BOOL FullScreen)
{
	HRESULT hRet;

    hRet = DirectDrawCreate(NULL, &lpDD, NULL);
    if (hRet != DD_OK)
        return InitFail(hWnd, hRet, "DirectDrawCreate FAILED");

  	if (FullScreen)
	{
			hRet = lpDD->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
			if (hRet != DD_OK)
				return InitFail(NULL, hRet, "SetCooperativeLevel FAILED");
			hRet = lpDD->SetDisplayMode(sizeX, sizeY, nbC);
			if (hRet != DD_OK)
				return InitFail(NULL, hRet, "SetDisplayMode FAILED");
	}
	else
	{
		hRet = lpDD->SetCooperativeLevel(hWnd, DDSCL_NORMAL);
		if (hRet != DD_OK)
			return InitFail(NULL, hRet, "SetCooperativeLevel FAILED");
	}

	hRet = CreatePrimary(hWnd);
	if (hRet != DD_OK)
		return InitFail(NULL, hRet, "Create Primary Failed");

	hRet = DirectInputCreate(hInst, DIRECTINPUT_VERSION, &lpdi, NULL); 
	if (hRet != DD_OK)
	{
		WNT = 1;
	}
	else
	{
		hRet = lpdi->CreateDevice(GUID_SysKeyboard, &lpdiDevice, NULL);
		if (hRet != DD_OK)
			return InitFail(NULL, hRet, "DInput Failed2");
		hRet = lpdiDevice->SetDataFormat(&c_dfDIKeyboard);  
		if (hRet != DD_OK)
			return InitFail(NULL, hRet, "DInput Failed3");
		hRet = lpdiDevice->SetCooperativeLevel(hWnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);  
		if (hRet != DD_OK)
			return InitFail(NULL, hRet, "DInput Failed4");
		if (lpdiDevice) lpdiDevice->Acquire();
		ZeroMemory (vnetbufferkeyb, 256);
		ZeroMemory (vIAbufferkeyb, 256);
	}

	hRet = DirectSoundCreate(NULL, &lpds, NULL);
    if (hRet == DD_OK)
	{
		Sound = 1;
		hRet = IDirectSound_SetCooperativeLevel(lpds, hWnd, DSSCL_PRIORITY);
		if (hRet != DD_OK)
			return InitFail (NULL, hRet, "DSound Failed");
		ZeroMemory(&dsbdesc, sizeof(DSBUFFERDESC));
		dsbdesc.dwSize = sizeof(DSBUFFERDESC);
		dsbdesc.dwFlags = DSBCAPS_PRIMARYBUFFER;
		hRet = lpds->CreateSoundBuffer(&dsbdesc, &lpdsbPrimary, NULL);
		if (hRet != DD_OK)
			return InitFail (NULL, hRet, "DSound Failed2");
		
		WAVEFORMATEX wfx;
		memset(&wfx, 0, sizeof(WAVEFORMATEX)); 
		wfx.wFormatTag = WAVE_FORMAT_PCM;
		wfx.nChannels = 2; 
		wfx.nSamplesPerSec = 22050;
		wfx.wBitsPerSample = 16; 
		wfx.nBlockAlign = wfx.wBitsPerSample / 8 * wfx.nChannels;
		wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign; 
		hRet = lpdsbPrimary->SetFormat(&wfx);  	
		if (hRet != DD_OK)
			return InitFail (NULL, hRet, "DSound Failed3");
		if (!LoadSounds ())
		{
			Sound = 0;
			lpds->Release ();
			lpds = NULL;
		}
	}

	return DD_OK;
}

HRESULT CreatePrimary(HWND &hWnd)
{
	DDSURFACEDESC ddsd;
    DDSCAPS ddscaps;
	HRESULT hRet;

    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
    ddsd.dwBackBufferCount = 1;
    hRet = lpDD->CreateSurface(&ddsd, &lpDDSPrimary, NULL);
    if (hRet != DD_OK)
	{
		DD_Error_Msg(NULL, hRet);
        return InitFail(NULL, hRet, "CreatePrimarySurface FAILED");
	}

    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
    hRet = lpDDSPrimary->GetAttachedSurface(&ddscaps, &lpDDSBack);
    if (hRet != DD_OK)
        return InitFail(NULL, hRet, "GetAttachedSurface FAILED");

	return DD_OK;
}

