#include "map.h"
#include "ddutil.h"

#include <stdio.h>
#include "time.h"
#include "ddutil.h"
#include "reseau.h"
#include "setup.h"
#include "alphabet.h"

void CreateDalleSurface(LPDIRECTDRAWSURFACE &lpDDSSpr)
{
	DDSURFACEDESC ddsd;

    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
	ddsd.dwWidth = DALLEX;
	ddsd.dwHeight = DALLEY;
    lpDD->CreateSurface(&ddsd, &lpDDSSpr, NULL);
}


MapSprite::MapSprite()
{
	LPDIRECTDRAWSURFACE lpDDStmp = DDLoadBitmap(lpDD, MAPOPTIONF, 0, 0);
	DDSURFACEDESC tmp;
	ZeroMemory (&tmp, sizeof (DDSURFACEDESC));
	tmp.dwSize = sizeof (DDSURFACEDESC);
	lpDDStmp->GetSurfaceDesc (&tmp);
	
	if (lpDDStmp)
	{
		RECT r = {0, 0, DALLEX, DALLEY};

		for (int i = 0 ; i < NUM_OPTION ; i++)
		{
			CreateDalleSurface(option[i]);
			option[i]->BltFast(0,0, lpDDStmp, &r, 0);
			DDSetColorKey(option[i], COLORKEY);
			r.left += DALLEX + 1; r.right+= DALLEX + 1;
			if ((DWORD)r.right > tmp.dwWidth)
			{
				r.left = 0; r.right = DALLEX - 1;
				r.top += DALLEY + 1; r.bottom += DALLEY + 1;
			}
		}
		lpDDStmp->Release();
		lpDDStmp = NULL;
	}
}

MapSprite::~MapSprite()
{
	for (int i = 0 ; i < NUM_OPTION ; i++)
	{
		option[i]->Release();
		option[i] = NULL;
	}
}


int LoadCarte(char *filename)
{
	FILE *f = NULL;

	f = fopen(filename, "rb");
	if (!f)
		return 0;
	fread(Carte, sizeof(Carte[0][0]), NBDALLEX*NBDALLEY, f);
	fclose(f);

	if (!reseau_enabled || (reseau_enabled && isHost))
		AddRandom();

	if (!MapSpr)
		MapSpr = new MapSprite();

	LoadAlphabet ();
	return 1;
}

void AddRandom()
{
	int nbbombesup=0, nbbombeinf=0,
		nbspeedup=0, nbspeeddown=0,
		nbflammeup=0, nbflammedown=0;
	srand( (unsigned)time( NULL ) );
	for (int i = 0; i<NBDALLEX; i++)
		for (int j = 0; j<NBDALLEY; j++)
		{
			if ((i==1 && j==1) ||
				(i==2 && j==1) ||
				(i==1 && j==2) ||
				(i==NBDALLEX-2 && j==1) ||
				(i==NBDALLEX-3 && j==1) ||
				(i==NBDALLEX-2 && j==2) ||
				(j==NBDALLEY-2 && i==1) ||
				(j==NBDALLEY-3 && i==1) ||
				(j==NBDALLEY-2 && i==2) ||
				(i==NBDALLEX-2 && j==NBDALLEY-2) ||
				(i==NBDALLEX-3 && j==NBDALLEY-2) ||
				(i==NBDALLEX-2 && j==NBDALLEY-3) ||
				(i==NBDALLEX/2 && j==1) ||
				(i==NBDALLEX/2+1 && j==1) ||
				(i==NBDALLEX/2+1 && j==2) ||
				(i==NBDALLEX/2 && j==NBDALLEY-2) ||
				(i==NBDALLEX/2+1 && j==NBDALLEY-2) ||
				(i==NBDALLEX/2+1 && j==NBDALLEY-3) ||
				(i==1 && j==NBDALLEY/2) ||
				(i==1 && j==NBDALLEY/2+1) ||
				(i==2 && j==NBDALLEY/2) ||
				(i==NBDALLEX-2 && j==NBDALLEY/2) ||
				(i==NBDALLEX-2 && j==NBDALLEY/2+1) ||
				(i==NBDALLEX-3 && j==NBDALLEY/2))
				continue;
			if (Carte[i][j] & TRAVERSABLE)
			{
				if (rand() > P_CENT)
				{
					Carte[i][j] &= ~TRAVERSABLE;
					Carte[i][j] |= DESTRUCTIBLE;
					if (rand() > 30000 && nbbombesup<=NBBOMBESUP)
					{
						Carte[i][j] |= BOMBESUP;
						nbbombesup++;
					}
					else
					if (rand() > 30000 && nbbombeinf<=NBBOMBEINF)
					{
						Carte[i][j] |= BOMBEINF;
						nbbombeinf++;
					}
					else
					if (rand() > 30000 && nbspeedup<=NBSPEEDUP)
					{
						Carte[i][j] |= SPEEDUP;
						nbspeedup++;
					}
					else
					if (rand() > 30000 && nbspeeddown<=NBSPEEDDOWN)
					{
						Carte[i][j] |= SPEEDDOWN;
						nbspeeddown++;
					}
					else
					if (rand() > 30000 && nbflammeup<=NBFLAMMEUP)
					{
						Carte[i][j] |= FLAMMEUP;
						nbflammeup++;
					}
					else
					if (rand() > 30000 && nbflammedown<=NBFLAMMEDOWN)
					{
						Carte[i][j] |= FLAMMEDOWN;
						nbflammedown++;
					}
				}
			}
		}
}

void DrawOption()
{
	for (int i = 0; i<NBDALLEX; i++)
		for (int ii = 0; ii<NBDALLEY; ii++)
			for (int iii = 1, j = 2 ; iii < NUM_OPTION ; j<<=1, iii++)
				if (Carte[i][ii] & j)
				{
					lpDDSBack->BltFast(i*DALLEX, ii*DALLEY, MapSpr->option[iii], NULL, DDBLTFAST_WAIT | DDBLTFAST_SRCCOLORKEY);
					break;
				}
}